@@ -1,52 +1,41 @@
 
 namespace FSpot {
 	public class PhotoImageView : ImageView {
-		public PhotoImageView (PhotoQuery query)
-		{
-			loader = new FSpot.AsyncPixbufLoader ();
-			loader.AreaUpdated += HandlePixbufAreaUpdated;
-			loader.AreaPrepared += HandlePixbufPrepared;
-			this.SizeAllocated += HandleSizeAllocated;
-			this.KeyPressEvent += HandleKeyPressEvent;
-			this.ScrollEvent += HandleScrollEvent;
-			this.Destroyed += HandleDestroy;
-			this.Query = query;
-		}
-		
 		public static double ZoomMultipler = 1.1;
 
 		public delegate void PhotoChangedHandler (PhotoImageView view);
 		public event PhotoChangedHandler PhotoChanged;
 		
-		private int current_photo = -1;
-		public int CurrentPhoto {
+		protected BrowsablePointer item;
+		
+		public PhotoImageView (IBrowsableCollection query)
+		{
+			loader = new FSpot.AsyncPixbufLoader ();
+			loader.AreaUpdated += HandlePixbufAreaUpdated;
+			loader.AreaPrepared += HandlePixbufPrepared;
+			loader.Done += HandleDone;
+
+			this.SizeAllocated += HandleSizeAllocated;
+			this.KeyPressEvent += HandleKeyPressEvent;
+			this.ScrollEvent += HandleScrollEvent;
+			query.ItemChanged += HandleQueryItemChanged;
+			this.item = new BrowsablePointer (query, -1);
+			item.IndexChanged += PhotoIndexChanged;
+			this.Destroyed += HandleDestroyed;
+		}
+		
+		new public BrowsablePointer Item {
 			get {
-				return current_photo;
-			}
-			set {
-				if (current_photo == value && this.Pixbuf != null){
-					return;
-				} else {
-					current_photo = value;
-					this.PhotoIndexChanged ();
-				}
+				return item;
 			}
 		}
 
-		public Photo Photo {
+		private IBrowsableCollection query;
+		public IBrowsableCollection Query {
 			get {
-				if (CurrentPhotoValid ()) 
-					return this.Query.Photos [CurrentPhoto];
-				else 
-					return null;
-			}
-		}
-
-		private PhotoQuery query;
-		public PhotoQuery Query {
-			get {
-				return query;
+				return item.Collection;
 			}
+#if false
 			set {
 				if (query != null) {
 					query.Changed -= HandleQueryChanged;
@@ -57,6 +46,7 @@ namespace FSpot {
 				query.Changed += HandleQueryChanged;
 				query.ItemChanged += HandleQueryItemChanged;
 			}
+#endif
 		}
 
 		public Gdk.Pixbuf CompletePixbuf ()
@@ -67,39 +57,26 @@ namespace FSpot {
 
 		public void Reload ()
 		{
-			if (!CurrentPhotoValid ())
+			if (!Item.IsValid)
 				return;
 			
-			int idx = CurrentPhoto;
-			CurrentPhoto = -1;
-			CurrentPhoto = idx;
+			PhotoIndexChanged (Item, null);
+			
 		}
-
+		/*
 		private void HandleQueryChanged (IBrowsableCollection browsable)
 		{
 			if (query == browsable)
 				Reload ();
 		}
+		*/
 
 		public void HandleQueryItemChanged (IBrowsableCollection browsable, int item)
 		{
-			if (item == CurrentPhoto)
+			if (item == Item.Index)
 				Reload ();
 		}
 
-		public bool CurrentPhotoValid ()
-		{
-			if (query == null ||
-			    query.Photos.Length == 0 ||
-			    CurrentPhoto >= Query.Photos.Length ||
-			    CurrentPhoto < 0) {
-				//System.Console.WriteLine ("Invalid CurrentPhoto");
-				return false;
-			}
-
-			return true;
-		}
-
 		// Display.
 		private void HandlePixbufAreaUpdated (object sender, Gdk.Rectangle area)
 		{
@@ -119,8 +96,8 @@ namespace FSpot {
 				next.Fill (0x00000000);
 #endif
 #if true
+			System.Uri uri = Item.Current.DefaultVersionUri;
 			try {
-				System.Uri uri = Photo.DefaultVersionUri;
 				Gdk.Pixbuf thumb = new Gdk.Pixbuf (ThumbnailGenerator.ThumbnailPath (uri));
 				if (thumb != null && next != null)
 					thumb.Composite (next, 0, 0,
@@ -136,7 +113,9 @@ namespace FSpot {
 					thumb.Dispose ();
 				}
 			} catch (System.Exception e) {
-				System.Console.WriteLine (e.ToString ());
+				FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);	
+				if (!(e is GLib.GException)) 
+					System.Console.WriteLine (e.ToString ());
 			}
 #endif
 
@@ -147,6 +126,42 @@ namespace FSpot {
 			this.ZoomFit ();
 		}
 
+		private void HandleDone (object sender, System.EventArgs args)
+		{
+			// FIXME the error hander here needs to provide proper information and we should
+			// pass the state and the write exception in the args
+			Gdk.Pixbuf prev = this.Pixbuf;
+			if (loader.Pixbuf == null) {
+				// FIXME in some cases the image passes completely through the
+				// pixbuf loader without properly loading... I'm not sure what to do about this other
+				// than try to load the image one last time.
+				this.Pixbuf = null;
+				if (!loader.Loading) {
+					try {
+						System.Console.WriteLine ("Falling back to file loader");
+
+						this.Pixbuf = FSpot.PhotoLoader.Load (item.Collection, 
+										      item.Index);
+					} catch (System.Exception e) {
+						if (!(e is GLib.GException))
+							System.Console.WriteLine (e.ToString ());
+					}
+				}
+
+				if (this.Pixbuf == null)
+					this.Pixbuf = new Gdk.Pixbuf (PixbufUtils.ErrorPixbuf, 0, 0, 
+								      PixbufUtils.ErrorPixbuf.Width, 
+								      PixbufUtils.ErrorPixbuf.Height);
+
+				this.ZoomFit ();
+			} else {
+				this.Pixbuf = loader.Pixbuf;
+			}
+
+			if (prev != this.Pixbuf && prev != null)
+				prev.Dispose ();
+		}		
+		
 		private bool fit = true;
 		public bool Fit {
 			get {
@@ -183,31 +198,47 @@ namespace FSpot {
 		FSpot.AsyncPixbufLoader loader;
 		FSpot.AsyncPixbufLoader next_loader;
 
-		private void PhotoIndexChanged () 
+		private void LoadErrorImage (System.Exception e)
 		{
-			if (!CurrentPhotoValid ())
+			// FIXME we should check the exception type and do something
+			// like offer the user a chance to locate the moved file and
+			// update the db entry, but for now just set the error pixbuf
+			
+			Gdk.Pixbuf old = this.Pixbuf;
+			this.Pixbuf = new Gdk.Pixbuf (PixbufUtils.ErrorPixbuf, 0, 0, 
+						      PixbufUtils.ErrorPixbuf.Width, 
+						      PixbufUtils.ErrorPixbuf.Height);
+			if (old != null)
+				old.Dispose ();
+			
+			this.ZoomFit ();
+		}
+
+		private void PhotoIndexChanged (BrowsablePointer item, IBrowsableItem old_item) 
+		{
+			// If it is just the position that changed fall out
+			if (old_item != null && Item.IsValid && this.Item.Current.DefaultVersionUri == old_item.DefaultVersionUri)
 				return;
 
 			if (load_async) {
 				try {
-					loader.Load (Photo.DefaultVersionPath);
-				} catch (System.Exception e) {
-					// FIXME we should check the exception type and do something
-					// like offer the user a chance to locate the moved file and
-					// update the db entry, but for now just set the error pixbuf
+					if (Item.IsValid)
+						loader.Load (Item.Current.DefaultVersionUri.LocalPath);
+					else
+						LoadErrorImage (null);
 
-					Gdk.Pixbuf old = this.Pixbuf;
-					this.Pixbuf = new Gdk.Pixbuf (PixbufUtils.ErrorPixbuf, 0, 0, 
-								      PixbufUtils.ErrorPixbuf.Width, 
-								      PixbufUtils.ErrorPixbuf.Height);
-					if (old != null)
-						old.Dispose ();
+				} catch (System.Exception e) {
+					System.Console.WriteLine (e.ToString ());
+					LoadErrorImage (e);
 				}
 			} else {	
 				Gdk.Pixbuf old = this.Pixbuf;
-				this.Pixbuf = FSpot.PhotoLoader.Load (Query, current_photo);
+				this.Pixbuf = FSpot.PhotoLoader.Load (item.Collection, 
+								      item.Index);
 				if (old != null)
 					old.Dispose ();
+
+				this.ZoomFit ();
 			}
 			
 			this.UnsetSelection ();
@@ -221,8 +252,6 @@ namespace FSpot {
 			Gdk.Pixbuf pixbuf = this.Pixbuf;
 			Gtk.ScrolledWindow scrolled = this.Parent as Gtk.ScrolledWindow;
 			
-			//System.Console.WriteLine ("ZoomFit");
-
 			if (pixbuf == null)
 				return;
 
@@ -251,43 +280,6 @@ namespace FSpot {
 				scrolled.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic);
 		}
 
-		public void Next () {
-			if (CurrentPhoto + 1 < query.Photos.Length)
-				CurrentPhoto++;
-			else
-				CurrentPhoto = 0;
-		}
-		
-		public void Prev () 
-		{
-			if (CurrentPhoto > 0)
-				CurrentPhoto --;
-			else
-				CurrentPhoto = query.Photos.Length - 1;
-		}
-		
-		public void First ()
-		{
-			if (query.Photos.Length > 0)
-				CurrentPhoto = 0;
-			else
-				CurrentPhoto = -1;
-		}
-
-		public void Last ()
-		{
-			if (query.Photos.Length > 0)
-				CurrentPhoto = query.Photos.Length -1;
-			else
-				CurrentPhoto = -1;
-		}
-
-		protected override void OnDestroyed ()
-		{
-			System.Console.WriteLine ("I'm feeling better");
-			base.OnDestroyed ();
-		}
-
 		[GLib.ConnectBefore]
 		private void HandleKeyPressEvent (object sender, Gtk.KeyPressEventArgs args)
 		{
@@ -297,18 +289,18 @@ namespace FSpot {
 			switch (args.Event.Key) {
 			case Gdk.Key.Page_Up:
 			case Gdk.Key.KP_Page_Up:
-				this.Prev ();
+				this.Item.MovePrevious ();
 				break;
 			case Gdk.Key.Home:
-				this.First ();
+				this.Item.Index = 0;
 				break;
 			case Gdk.Key.End:
-				this.Last ();
+				this.Item.Index = this.Query.Count - 1;
 				break;
 			case Gdk.Key.space:
 			case Gdk.Key.Page_Down:
 			case Gdk.Key.KP_Page_Down:
-				this.Next ();
+				this.Item.MoveNext ();
 				break;
 			case Gdk.Key.Key_0:
 				this.Fit = true;
@@ -343,17 +335,11 @@ namespace FSpot {
 			this.Fit = false;
 		}
 		
-		private void HandleDestroy (object sender, System.EventArgs args)
+		private void HandleDestroyed (object sender, System.EventArgs args)
 		{
 			//loader.AreaUpdated -= HandlePixbufAreaUpdated;
 			//loader.AreaPrepared -= HandlePixbufPrepared;
 			loader.Dispose ();
 		}
-
-		protected override bool OnDestroyEvent (Gdk.Event evnt)
-		{
-			System.Console.WriteLine ("I'm feeling better");
-			return base.OnDestroyEvent (evnt);
-		}
 	}
 }
